@@ -132,20 +132,14 @@ ev_annotation_window_sync_contents (EvAnnotationWindow *window)
 
 static void
 ev_annotation_window_set_color (EvAnnotationWindow *window,
-				GdkColor           *color)
+				GdkRGBA            *color)
 {
         GtkStyleProperties *properties;
         GtkStyleProvider   *provider;
-	GdkRGBA             rgba;
-
-        rgba.red = color->red / 65535.;
-        rgba.green = color->green / 65535.;
-        rgba.blue = color->blue / 65535.;
-        rgba.alpha = 1;
 
         properties = gtk_style_properties_new ();
         gtk_style_properties_set (properties, 0,
-                                  "background-color", &rgba,
+                                  "background-color", color,
                                   NULL);
 
         provider = GTK_STYLE_PROVIDER (properties);
@@ -160,6 +154,14 @@ ev_annotation_window_set_color (EvAnnotationWindow *window,
         g_object_unref (properties);
 }
 
+static void
+ev_annotation_window_set_opacity (EvAnnotationWindow *window,
+		                  gdouble             opacity)
+{
+	gtk_widget_set_opacity (GTK_WIDGET (window), opacity);
+	gtk_widget_set_opacity (GTK_WIDGET (window->text_view), opacity);
+}
+
 static void
 ev_annotation_window_label_changed (EvAnnotationMarkup *annot,
 				    GParamSpec         *pspec,
@@ -176,10 +178,21 @@ ev_annotation_window_color_changed (EvAnnotation       *annot,
 				    GParamSpec         *pspec,
 				    EvAnnotationWindow *window)
 {
-	GdkColor color;
+	GdkRGBA rgba;
 
-	ev_annotation_get_color (annot, &color);
-	ev_annotation_window_set_color (window, &color);
+	ev_annotation_get_rgba (annot, &rgba);
+	ev_annotation_window_set_color (window, &rgba);
+}
+
+static void
+ev_annotation_window_opacity_changed (EvAnnotation       *annot,
+		                      GParamSpec         *pspec,
+				      EvAnnotationWindow *window)
+{
+	gdouble opacity;
+
+	opacity = ev_annotation_markup_get_opacity (EV_ANNOTATION_MARKUP (annot));
+	ev_annotation_window_set_opacity (window, opacity);
 }
 
 static void
@@ -391,9 +404,10 @@ ev_annotation_window_constructor (GType                  type,
 	EvAnnotationMarkup *markup;
 	const gchar        *contents;
 	const gchar        *label;
-	GdkColor            color;
+	GdkRGBA             color;
 	EvRectangle        *rect;
 	gdouble             scale;
+	gdouble             opacity;
 
 	object = G_OBJECT_CLASS (ev_annotation_window_parent_class)->constructor (type,
 										  n_construct_properties,
@@ -417,8 +431,12 @@ ev_annotation_window_constructor (GType                  type,
 			   (gint)((rect->x2 - rect->x1) * scale),
 			   (gint)((rect->y2 - rect->y1) * scale));
 
-	ev_annotation_get_color (annot, &color);
+	ev_annotation_get_rgba (annot, &color);
 	ev_annotation_window_set_color (window, &color);
+
+	opacity = ev_annotation_markup_get_opacity (markup);
+	ev_annotation_window_set_opacity (window, opacity);
+
 	gtk_widget_set_name (GTK_WIDGET (window), ev_annotation_get_name (annot));
 	gtk_window_set_title (GTK_WINDOW (window), label);
 	gtk_label_set_text (GTK_LABEL (window->title), label);
@@ -434,9 +452,12 @@ ev_annotation_window_constructor (GType                  type,
 	g_signal_connect (annot, "notify::label",
 			  G_CALLBACK (ev_annotation_window_label_changed),
 			  window);
-	g_signal_connect (annot, "notify::color",
+	g_signal_connect (annot, "notify::rgba",
 			  G_CALLBACK (ev_annotation_window_color_changed),
 			  window);
+	g_signal_connect (annot, "notify::opacity",
+			  G_CALLBACK (ev_annotation_window_opacity_changed),
+			  window);
 
 	return object;
 }
