@@ -72,6 +72,12 @@ typedef struct {
 	EvImage *image;
 } ImageDNDInfo;
 
+typedef enum {
+	EV_PAN_ACTION_NONE,
+	EV_PAN_ACTION_NEXT,
+	EV_PAN_ACTION_PREV
+} EvPanAction;
+
 /* Annotation popup windows */
 typedef struct {
 	GtkWidget *window;
@@ -222,6 +228,12 @@ struct _EvView {
 	gboolean cursor_visible;
 	guint    cursor_blink_timeout_id;
 	guint    cursor_blink_time;
+
+	/* Gestures */
+	GtkGesture *pan_gesture;
+	GtkGesture *zoom_gesture;
+	gdouble prev_zoom_gesture_scale;
+	EvPanAction pan_action;
 };
 
 struct _EvViewClass {
@@ -241,6 +253,8 @@ struct _EvViewClass {
 				       EvSourceLink   *link);
         void     (*annot_added)       (EvView         *view,
 				       EvAnnotation   *annot);
+        void     (*annot_removed)     (EvView         *view,
+				       EvAnnotation   *annot);
         void     (*layers_changed)    (EvView         *view);
         gboolean (*move_cursor)       (EvView         *view,
 				       GtkMovementStep step,
